"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Selector = void 0;
const internal_1 = require("../internal");
const persist_1 = require("../state/persist");
class Selector extends internal_1.Computed {
    constructor(collection, id, config) {
        // If no key provided set it to dummy (dummyKey)
        if (!id)
            id = 'dummy';
        // Instantiate Computed with 'computed' function
        super(collection.agileInstance(), () => findData(collection, id));
        if (config === null || config === void 0 ? void 0 : config.key)
            this._key = config === null || config === void 0 ? void 0 : config.key;
        this.collection = () => collection;
        this._id = id;
        // Set type of State to object because a collection item is always an object
        this.type(Object);
    }
    set id(val) {
        this.select(val);
    }
    get id() {
        return this._id;
    }
    //=========================================================================================================
    // Select
    //=========================================================================================================
    /**
     * Changes the id on which the selector is watching
     */
    select(id, options) {
        var _a;
        // Assign defaults to config
        options = internal_1.defineConfig(options, {
            background: false,
            sideEffects: true
        });
        // Remove item if its a placeholder because if so it won't be needed without being selected by this selector
        if ((_a = this.collection().data[this.id]) === null || _a === void 0 ? void 0 : _a.isPlaceholder)
            delete this.collection().data[this.id];
        // Set _id to new id
        this._id = id;
        // Update Computed Function with new key(id)
        this.updateComputeFunction(() => findData(this.collection(), id), [], options);
        return this;
    }
    //=========================================================================================================
    // Overwrite Persist
    //=========================================================================================================
    /**
     * Saves the state in the local storage or in a own configured storage
     * @param key - the storage key (if no key passed it will take the state key)
     */
    persist(key) {
        persist_1.persistValue(this, key);
        return this;
    }
    //=========================================================================================================
    // Overwrite getPerstiableValue
    //=========================================================================================================
    /**
     * @internal
     *  Will return the perstiable Value of this state..
     */
    getPersistableValue() {
        return this.id;
    }
}
exports.Selector = Selector;
//=========================================================================================================
// Find Data
//=========================================================================================================
/**
 * Computed function for the Selector
 */
function findData(collection, id) {
    // Find data by id in collection
    let item = collection.findById(id);
    // If data is not found, create placeholder item, so that when real data is collected it maintains connection and causes a rerender
    if (!item) {
        const newItem = new internal_1.Item(collection, { id: id });
        newItem.isPlaceholder = true;
        collection.data[id] = newItem;
        item = newItem;
    }
    // If initial State is still {id: id}.. because of placeholder item and the value isn't {id: id} -> had got real value.. set the initial State to this first real value
    if (JSON.stringify(item.initialState) === JSON.stringify({ id: id }) && JSON.stringify(item.nextState) !== JSON.stringify({ id: id })) {
        item.initialState = internal_1.copy(item.nextState);
        item.previousState = internal_1.copy(item.nextState);
    }
    // Have to create the final Value here, to get added to the track states also if the item doesn't exist yet.. (otherwise auto tracking state wouldn't work -> this won't get called if the item changes )
    const finalValue = item.value;
    // If item doesn't exist return undefined.. otherwise it would return {id: id}
    if (!item.exists)
        return undefined;
    return finalValue;
}
