"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Computed = void 0;
const internal_1 = require("../internal");
class Computed extends internal_1.State {
    constructor(agileInstance, computeFunction, deps = []) {
        super(agileInstance, computeFunction());
        this.deps = [];
        this.hardCodedDeps = [];
        this.agileInstance = () => agileInstance;
        this.computeFunction = computeFunction;
        this.hardCodedDeps = deps;
        // Recompute for setting initial state value and adding missing dependencies
        this.recompute();
    }
    set value(value) {
        console.error('Agile: Can not mutate Computed value, please use recompute()');
    }
    get value() {
        return super.value;
    }
    //=========================================================================================================
    // Recompute
    //=========================================================================================================
    /**
     * Will call the computeFunction and update the dependencies
     */
    recompute(options) {
        // Assign defaults to config
        options = internal_1.defineConfig(options, {
            background: false,
            sideEffects: true
        });
        // Set State to nextState
        this.ingest(options);
    }
    //=========================================================================================================
    // Updates Compute Function
    //=========================================================================================================
    /**
     * Updates the Compute Function
     */
    updateComputeFunction(computeFunction, deps = [], options) {
        this.computeFunction = computeFunction;
        this.hardCodedDeps = deps;
        // Recompute for setting initial state value and adding missing dependencies
        this.recompute(options);
    }
    //=========================================================================================================
    // Compute Values
    //=========================================================================================================
    /**
     * @internal
     * Will add auto tracked dependencies to this and calls the computeFunction
     */
    computeValue() {
        // Set tracking state to true which will than track all states which for instance call state.value
        this.agileInstance().runtime.trackState = true;
        // Call computeFunction
        const computedValue = this.computeFunction();
        // Get tracked states and set trackSate to false
        let foundStates = this.agileInstance().runtime.getTrackedStates();
        // Handle foundStates dependencies
        const newDeps = [];
        foundStates.forEach(state => {
            // Add the state to newDeps
            newDeps.push(state);
            // Add this as dependency of the state
            state.dep.depend(this);
        });
        // Handle hardCoded dependencies
        this.hardCodedDeps.forEach(state => {
            // Add this as dependency of the state
            state.dep.depend(this);
        });
        // Set deps
        this.deps = [...this.hardCodedDeps, ...newDeps];
        return computedValue;
    }
    //=========================================================================================================
    // Overwriting some functions which can't be used in computed
    //=========================================================================================================
    patch() {
        console.error('Agile: can not use patch method on Computed since the value is dynamic!');
        return this;
    }
    persist(key) {
        console.error('Agile: Computed state can not be persisted since the value is dynamic!', key);
        return this;
    }
}
exports.Computed = Computed;
